@@ -21,7 +21,11 @@ import hypothesis.internal.conjecture.utils as cu
 from hypothesis.errors import InvalidArgument
 from hypothesis.internal.compat import OrderedDict
 from hypothesis.internal.conjecture.utils import combine_labels
-from hypothesis.searchstrategy.strategies import MappedSearchStrategy, SearchStrategy
+from hypothesis.searchstrategy.strategies import (
+    MappedSearchStrategy,
+    SearchStrategy,
+    filter_not_satisfied,
+)
 
 
 class TupleStrategy(SearchStrategy):
@@ -125,9 +129,9 @@ class ListStrategy(SearchStrategy):
 
 
 class UniqueListStrategy(ListStrategy):
-    def __init__(self, elements, min_size, max_size, key):
+    def __init__(self, elements, min_size, max_size, keys):
         super(UniqueListStrategy, self).__init__(elements, min_size, max_size)
-        self.key = key
+        self.keys = keys
 
     def do_draw(self, data):
         if self.element_strategy.is_empty:
@@ -140,16 +144,26 @@ class UniqueListStrategy(ListStrategy):
             max_size=self.max_size,
             average_size=self.average_size,
         )
-        seen = set()
+        seen_sets = tuple(set() for _ in self.keys)
         result = []
 
+        # We construct a filtered strategy here rather than using a check-and-reject
+        # approach because some strategies have special logic for generation under a
+        # filter, and FilteredStrategy can consolidate multiple filters.
+        filtered = self.element_strategy.filter(
+            lambda val: all(
+                key(val) not in seen for (key, seen) in zip(self.keys, seen_sets)
+            )
+        )
         while elements.more():
-            value = data.draw(self.element_strategy)
-            k = self.key(value)
-            if k in seen:
+            value = filtered.filtered_strategy.do_filtered_draw(
+                data=data, filter_strategy=filtered
+            )
+            if value is filter_not_satisfied:
                 elements.reject()
             else:
-                seen.add(k)
+                for key, seen in zip(self.keys, seen_sets):
+                    seen.add(key(value))
                 result.append(value)
         assert self.max_size >= len(result) >= self.min_size
         return result
@@ -174,7 +188,7 @@ class FixedKeysDictStrategy(MappedSearchStrategy):
             except TypeError:
                 self.keys = tuple(sorted(strategy_dict.keys(), key=repr))
         super(FixedKeysDictStrategy, self).__init__(
-            strategy=TupleStrategy((strategy_dict[k] for k in self.keys))
+            strategy=TupleStrategy(strategy_dict[k] for k in self.keys)
         )
 
     def calc_is_empty(self, recur):
